@@ -18,7 +18,7 @@ from os import path
 from pathlib import Path
 from functools import lru_cache
 from types import ModuleType, FunctionType, CodeType, FrameType
-from typing import Optional, Tuple, Union, List, MutableMapping, Callable
+from typing import Tuple, Union, List, Mapping, Callable
 
 from executing import Source
 
@@ -32,49 +32,51 @@ IgnoreElemType = Union[
     # the module (filename) and qualname
     # If module is None, then all qualname matches the 2nd element
     # will be ignored. Used to ignore <lambda> internally
-    Tuple[Optional[Union[ModuleType, str]], str],
+    Tuple[Union[ModuleType, str], str],
     # Function and number of its decorators
-    Tuple[FunctionType, int]
+    Tuple[FunctionType, int],
 ]
 IgnoreType = Union[IgnoreElemType, List[IgnoreElemType]]
 
-MODULE_IGNORE_ID_NAME = '__varname_ignore_id__'
+ArgSourceType = Union[ast.AST, str]
+ArgSourceType = Union[ArgSourceType, Tuple[ArgSourceType, ...]]
+ArgSourceType = Union[ArgSourceType, Mapping[str, ArgSourceType]]
 
-class config: # pylint: disable=invalid-name
+MODULE_IGNORE_ID_NAME = "__varname_ignore_id__"
+
+
+class config:  # pylint: disable=invalid-name
     """Global configurations for varname
 
     Attributes:
         debug: Show debug information for frames being ignored
     """
+
     debug = False
 
+
 class VarnameRetrievingError(Exception):
     """When failed to retrieve the varname"""
 
+
 class QualnameNonUniqueError(Exception):
     """When a qualified name is used as an ignore element but references to
     multiple objects in a module"""
 
+
 class NonVariableArgumentError(Exception):
     """When vars_only is True but try to retrieve name of
     a non-variable argument"""
 
-class ImproperUseError(VarnameRetrievingError):
+
+class ImproperUseError(Exception):
     """When varname() is improperly used"""
-    def __init__(self, message: str) -> None:
-        message = (
-            f"{message}\n\n"
-            "(Warning: `ImproperUseError` is a subclass of "
-            "`VarnameRetrievingError` for now for backward compatibility, "
-            "so you can still use `VarnameRetrievingError` to catch it. "
-            "But it will be independent of `VarnameRetrievingError` "
-            "in the future.)"
-        )
-        super().__init__(message)
+
 
 class MaybeDecoratedFunctionWarning(Warning):
     """When a suspecious decorated function used as ignore function directly"""
 
+
 class MultiTargetAssignmentWarning(Warning):
     """When varname tries to retrieve variable name in
     a multi-target assignment"""
@@ -85,12 +87,13 @@ def cached_getmodule(codeobj: CodeType):
     """Cached version of inspect.getmodule"""
     return inspect.getmodule(codeobj)
 
+
 def get_node(
-        frame: int,
-        ignore: Optional[IgnoreType] = None,
-        raise_exc: bool = True,
-        ignore_lambda: bool = True
-) -> Optional[ast.AST]:
+    frame: int,
+    ignore: IgnoreType = None,
+    raise_exc: bool = True,
+    ignore_lambda: bool = True,
+) -> ast.AST:
     """Try to get node from the executing object.
 
     This can fail when a frame is failed to retrieve.
@@ -100,18 +103,19 @@ def get_node(
     When the node can not be retrieved, try to return the first statement.
     """
     from .ignore import IgnoreList
+
     ignore = IgnoreList.create(ignore, ignore_lambda=ignore_lambda)
     try:
-        frame = ignore.get_frame(frame)
+        frameobj = ignore.get_frame(frame)
     except VarnameRetrievingError:
         return None
 
-    return get_node_by_frame(frame, raise_exc)
+    return get_node_by_frame(frameobj, raise_exc)
+
 
 def get_node_by_frame(
-        frame: FrameType,
-        raise_exc: bool = True
-) -> Optional[ast.AST]:
+    frame: FrameType, raise_exc: bool = True
+) -> ast.AST:
     """Get the node by frame, raise errors if possible"""
     exect = Source.executing(frame)
 
@@ -127,16 +131,18 @@ def get_node_by_frame(
 
     return None
 
-def lookfor_parent_assign(node: ast.AST) -> Optional[ast.Assign]:
+
+def lookfor_parent_assign(node: ast.AST) -> Union[ast.Assign, ast.AnnAssign]:
     """Look for an ast.Assign node in the parents"""
-    while hasattr(node, 'parent'):
+    while hasattr(node, "parent"):
         node = node.parent
 
         if isinstance(node, (ast.AnnAssign, ast.Assign)):
             return node
     return None
 
-def node_name(node: ast.AST) -> Optional[Union[str, Tuple[Union[str, tuple]]]]:
+
+def node_name(node: ast.AST) -> Union[str, Tuple[Union[str, Tuple], ...]]:
     """Get the node node name.
 
     Raises ImproperUseError when failed
@@ -153,6 +159,7 @@ def node_name(node: ast.AST) -> Optional[Union[str, Tuple[Union[str, tuple]]]]:
         f"not {ast.dump(node)}"
     )
 
+
 @lru_cache()
 def bytecode_nameof(code: CodeType, offset: int) -> str:
     """Cached Bytecode version of nameof
@@ -163,16 +170,13 @@ def bytecode_nameof(code: CodeType, offset: int) -> str:
     manipulated to run but sourcecode is not available.
     """
     instructions = list(dis.get_instructions(code))
-    (current_instruction_index, current_instruction), = (
+    ((current_instruction_index, current_instruction),) = (
         (index, instruction)
         for index, instruction in enumerate(instructions)
         if instruction.offset == offset
     )
 
-    if current_instruction.opname in (
-            "CALL_FUNCTION_EX",
-            "CALL_FUNCTION_KW"
-    ):
+    if current_instruction.opname in ("CALL_FUNCTION_EX", "CALL_FUNCTION_KW"):
         raise VarnameRetrievingError(
             "'nameof' can only be called with a single positional argument "
             "when source code is not avaiable."
@@ -181,9 +185,7 @@ def bytecode_nameof(code: CodeType, offset: int) -> str:
     if current_instruction.opname not in ("CALL_FUNCTION", "CALL_METHOD"):
         raise VarnameRetrievingError("Did you call 'nameof' in a weird way?")
 
-    name_instruction = instructions[
-        current_instruction_index - 1
-    ]
+    name_instruction = instructions[current_instruction_index - 1]
 
     if not name_instruction.opname.startswith("LOAD_"):
         raise VarnameRetrievingError("Argument must be a variable or attribute")
@@ -198,6 +200,7 @@ def bytecode_nameof(code: CodeType, offset: int) -> str:
 
     return name
 
+
 def attach_ignore_id_to_module(module: ModuleType) -> None:
     """Attach the ignore id to module
 
@@ -209,30 +212,28 @@ def attach_ignore_id_to_module(module: ModuleType) -> None:
     the module. Since this probably means the source is not avaiable and
     `inspect.getmodule` would not work
     """
-    module_file = getattr(module, '__file__', None)
+    module_file = getattr(module, "__file__", None)
     if module_file is not None and path.isfile(module_file):
         return
     # or it's already been set
     if hasattr(module, MODULE_IGNORE_ID_NAME):
         return
 
-    setattr(module, MODULE_IGNORE_ID_NAME, f'<varname-ignore-{id(module)})')
+    setattr(module, MODULE_IGNORE_ID_NAME, f"<varname-ignore-{id(module)})")
 
 
 def frame_matches_module_by_ignore_id(
-        frame: FrameType,
-        module: ModuleType
+    frame: FrameType, module: ModuleType
 ) -> bool:
     """Check if the frame is from the module by ignore id"""
     ignore_id_attached = getattr(module, MODULE_IGNORE_ID_NAME, object())
     ignore_id_from_frame = frame.f_globals.get(MODULE_IGNORE_ID_NAME, object())
     return ignore_id_attached == ignore_id_from_frame
 
+
 @lru_cache()
 def check_qualname_by_source(
-        source: Source,
-        modname: str,
-        qualname: str
+    source: Source, modname: str, qualname: str
 ) -> None:
     """Check if a qualname in module is unique"""
     if not source.tree:
@@ -245,9 +246,10 @@ def check_qualname_by_source(
             f"{modname!r} refers to multiple objects."
         )
 
+
 def debug_ignore_frame(
-        msg: str,
-        frameinfo: Optional[inspect.FrameInfo] = None
+    msg: str,
+    frameinfo: inspect.FrameInfo = None
 ) -> None:
     """Print the debug message for a given frame info object
 
@@ -258,13 +260,16 @@ def debug_ignore_frame(
     if not config.debug:
         return
     if frameinfo is not None:
-        msg = (f'{msg} [In {frameinfo.function!r} at '
-               f'{frameinfo.filename}:{frameinfo.lineno}]')
-    sys.stderr.write(f'[{__package__}] DEBUG: {msg}\n')
+        msg = (
+            f"{msg} [In {frameinfo.function!r} at "
+            f"{frameinfo.filename}:{frameinfo.lineno}]"
+        )
+    sys.stderr.write(f"[{__package__}] DEBUG: {msg}\n")
 
-def argnode_source(source: Source,
-                   node: ast.AST,
-                   vars_only: bool) -> Union[str, ast.AST]:
+
+def argnode_source(
+    source: Source, node: ast.AST, vars_only: bool
+) -> Union[str, ast.AST]:
     """Get the source of an argument node
 
     Args:
@@ -279,22 +284,25 @@ def argnode_source(source: Source,
     """
     if vars_only:
         return (
-            node.id if isinstance(node, ast.Name)
-            else node.attr if isinstance(node, ast.Attribute)
+            node.id
+            if isinstance(node, ast.Name)
+            else node.attr
+            if isinstance(node, ast.Attribute)
             else node
         )
 
     # requires asttokens
     return source.asttokens().get_text(node)
 
+
 @lru_cache()
 def get_argument_sources(
-        source: Source,
-        node: ast.Call,
-        func: Callable,
-        vars_only: bool,
-        pos_only: bool
-) -> MutableMapping[str, Union[ast.AST, str]]:
+    source: Source,
+    node: ast.Call,
+    func: Callable,
+    vars_only: bool,
+    pos_only: bool,
+) -> Mapping[str, ArgSourceType]:
     """Get the sources for argument from an ast.Call node
 
     >>> def func(a, b, c, d=4):
@@ -309,14 +317,17 @@ def get_argument_sources(
     signature = inspect.signature(func, follow_wrapped=False)
     # func(y, x, c=z)
     # ['y', 'x'], {'c': 'z'}
-    arg_sources = [argnode_source(source, argnode, vars_only)
-                   for argnode in node.args]
-    kwarg_sources = {
-        argnode.arg: argnode_source(source,
-                                    argnode.value,
-                                    vars_only)
-        for argnode in node.keywords
-    } if not pos_only else {}
+    arg_sources = [
+        argnode_source(source, argnode, vars_only) for argnode in node.args
+    ]
+    kwarg_sources = (
+        {
+            argnode.arg: argnode_source(source, argnode.value, vars_only)
+            for argnode in node.keywords
+        }
+        if not pos_only
+        else {}
+    )
     bound_args = signature.bind_partial(*arg_sources, **kwarg_sources)
     argument_sources = bound_args.arguments
     # see if *args and **kwargs have anything assigned
@@ -328,10 +339,8 @@ def get_argument_sources(
             argument_sources.setdefault(parameter.name, {})
     return argument_sources
 
-def get_function_called_argname(
-        frame: FrameType,
-        node: ast.AST
-) -> Callable:
+
+def get_function_called_argname(frame: FrameType, node: ast.AST) -> Callable:
     """Get the function who called argname"""
     # We need node to be ast.Call
     if not isinstance(node, ast.Call):
@@ -343,10 +352,9 @@ def get_function_called_argname(
     # variable
     if isinstance(node.func, ast.Name):
         func = frame.f_locals.get(
-            node.func.id,
-            frame.f_globals.get(node.func.id)
+            node.func.id, frame.f_globals.get(node.func.id)
         )
-        if func is None: # pragma: no cover
+        if func is None:  # pragma: no cover
             # not sure how it would happen but in case
             raise VarnameRetrievingError(
                 f"Cannot retrieve the function by {node.func.id!r}."
@@ -377,10 +385,11 @@ def get_function_called_argname(
         "passing the function to 'argname' explicitly."
     )
     expr = ast.Expression(node.func)
-    code = compile(expr, '<ast-call>', 'eval')
+    code = compile(expr, "<ast-call>", "eval")
     # pylint: disable=eval-used
     return eval(code, frame.f_globals, frame.f_locals)
 
+
 def parse_argname_subscript(node: ast.Subscript):
     """Parse the ast.Subscript node passed to argname
 
@@ -395,14 +404,16 @@ def parse_argname_subscript(node: ast.Subscript):
     if not isinstance(name, ast.Name):
         raise ValueError(f"Expect {ast.dump(name)} to be a variable.")
 
-    subscript = node.slice
+    subscript = node.slice # type: ast.AST
     if isinstance(subscript, ast.Index):
         subscript = subscript.value
     if not isinstance(subscript, (ast.Str, ast.Num, ast.Constant)):
         raise ValueError(f"Expect {ast.dump(subscript)} to be a constant.")
 
-    subscript = getattr(subscript, 'value', subscript) # ast.Index, ast.Constant
-    subscript = getattr(subscript, 'n', subscript) # ast.Num
-    subscript = getattr(subscript, 's', subscript) # ast.Str
+    subscript = getattr(
+        subscript, "value", subscript
+    )  # ast.Index, ast.Constant
+    subscript = getattr(subscript, "n", subscript)  # ast.Num
+    subscript = getattr(subscript, "s", subscript)  # ast.Str
 
     return name.id, subscript
